//
//  DigestAlgorithm.swift
//  Runner
//
//  Created by Lacuna Software on 5/13/20.
//

import Foundation
import CommonCrypto

public class DigestAlgorithm: BaseAlgorithm {
    
    public let lengthBytes: Int
    
    public static let md5 = DigestAlgorithm("MD5", "1.2.840.113549.2.5", 16)
    public static let sha1 = DigestAlgorithm("SHA-1", "1.3.14.3.2.26", 20)
    public static let sha224 = DigestAlgorithm("SHA-224", "2.16.840.1.101.3.4.2.4", 28)
    public static let sha256 = DigestAlgorithm("SHA-256", "2.16.840.1.101.3.4.2.1", 32)
    public static let sha384 = DigestAlgorithm("SHA-384", "2.16.840.1.101.3.4.2.2", 48)
    public static let sha512 = DigestAlgorithm("SHA-512", "2.16.840.1.101.3.4.2.3", 64)
    
    private static let allAlgorithms = [
        md5, sha1, sha224, sha256, sha384, sha512
    ]
    
    fileprivate init(_ name: String, _ oid: String, _ length: Int) {
        lengthBytes = length
        super.init(name, oid)
    }
    
    public static func parse(_ digestAlgorithm: String) throws -> DigestAlgorithm {
        return allAlgorithms.first(where: { (da: DigestAlgorithm) -> Bool in
            return digestAlgorithm == da.oid || canonString(value: digestAlgorithm) == da.name
        })!
    }
    
    public func computeHash(data: Data) -> Data {
        var hash = [UInt8](repeating: 0,  count: lengthBytes)
        data.withUnsafeBytes {
            switch oid {
                case DigestAlgorithm.md5.oid:
                    _ = CC_MD5($0.baseAddress, CC_LONG(data.count), &hash)
                    
                case DigestAlgorithm.sha1.oid:
                    _ = CC_SHA1($0.baseAddress, CC_LONG(data.count), &hash)
                    
                case DigestAlgorithm.sha224.oid:
                    _ = CC_SHA224($0.baseAddress, CC_LONG(data.count), &hash)
                    
                case DigestAlgorithm.sha256.oid:
                    _ = CC_SHA256($0.baseAddress, CC_LONG(data.count), &hash)
                    
                case DigestAlgorithm.sha384.oid:
                    _ = CC_SHA384($0.baseAddress, CC_LONG(data.count), &hash)
                    
                case DigestAlgorithm.sha512.oid:
                    _ = CC_SHA512($0.baseAddress, CC_LONG(data.count), &hash)
                default:
                    break
            }
        }
        return Data(hash)
    }
}
