//
//  DigestInfo.swift
//  Runner
//
//  Created by Lacuna Software on 5/13/20.
//

import Foundation

public class DigestInfo {
    
    static let digestIds: [String: Data] = [
        DigestAlgorithm.md5.oid:    Data(Array<UInt8>(arrayLiteral: 0x30, 0x20, 0x30, 0x0c, 0x06, 0x08, 0x2a, 0x86, 0x48, 0x86, 0xf7, 0x0d, 0x02, 0x05, 0x05, 0x00, 0x04, 0x10 )),
        DigestAlgorithm.sha1.oid:   Data(Array<UInt8>(arrayLiteral: 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03, 0x02, 0x1a, 0x05, 0x00, 0x04, 0x14 )),
        DigestAlgorithm.sha256.oid: Data(Array<UInt8>(arrayLiteral: 0x30, 0x31, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x01, 0x05, 0x00, 0x04, 0x20 )),
        DigestAlgorithm.sha384.oid: Data(Array<UInt8>(arrayLiteral: 0x30, 0x41, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x02, 0x05, 0x00, 0x04, 0x30 )),
        DigestAlgorithm.sha512.oid: Data(Array<UInt8>(arrayLiteral: 0x30, 0x51, 0x30, 0x0d, 0x06, 0x09, 0x60, 0x86, 0x48, 0x01, 0x65, 0x03, 0x04, 0x02, 0x03, 0x05, 0x00, 0x04, 0x40 )),
    ]
    
    public static func encode(digestAlgorithm: DigestAlgorithm, digestValue: Data) throws -> Data {
        let digestId = digestIds[digestAlgorithm.oid]
        if digestId == nil {
            throw CrossPkiError.error(.algorithmNotSupported, "Unknown algorithm: \(digestAlgorithm)")
        }
        if digestAlgorithm.lengthBytes != digestValue.count {
            throw CrossPkiError.error(ErrorCodes.internalError, "Invalid digest value length for algorithm \(digestAlgorithm): \(digestValue.count)")
        }
        var digestInfo = Data(digestId!)
        digestInfo.append(digestValue);
        return digestInfo
    }
}
